import { storeToRefs } from 'pinia'
import { ThemeEnum } from '~/constants'
import type { MaybeElementRef } from '~/utils'
import { generateColors, setCssVar, toggleClass } from '~/utils'

const HEADER_HEIGHT = '--header-height'
// const HEADER_BG_COLOR_VAR = '--header-background-color'
// const HEADER_TEXT_COLOR_VAR = '--header-text-color'
// const HEADER_ACTION_HOVER_BG_COLOR_VAR = '--header-action-hover-bg-color'

const ASIDE_WIDTH = '--aside-width'
// const ASIDE_DARK_BG_COLOR = '--aside-background-color'
// const ASIDE_TEXT_COLOR_VAR = '--aside-text-color'

// const TRIGGER_BG_COLOR_VAR = '--trigger-background-color'

const TAB_BAR_HEIGHT = '--tab-bar-height'

const FOOTER_HEIGHT = '--footer-height'

// const LIGHT_TEXT_COLOR = 'rgba(0,0,0,.85)'
// const DARK_TEXT_COLOR = '#fff'

export function createMediaPrefersColorSchemeListen() {
  const { setAppConfig } = useAppConfig()

  // 监听系统主题变更
  useEventListener(
    window.matchMedia('(prefers-color-scheme: dark)'),
    'change',
    (e: MediaQueryListEvent) => setAppConfig({ theme: e.matches ? ThemeEnum.DARK : ThemeEnum.LIGHT }),
  )
}

function toggleGrayMode(val: boolean) {
  toggleClass(val, 'gray-mode', document.documentElement)
}

function toggleColorWeak(val: boolean) {
  toggleClass(val, 'color-weak', document.documentElement)
}

export function createGridLayoutListen(el: MaybeElementRef | null) {
  const {
    isTopMenu,
    sidebar,
    header,
    footer,
    tabTar,
    getCollapsedShowTitle,
    menu,
    isMixSidebar,
  } = useAppConfig()
  const asideWidth = useCssVar(ASIDE_WIDTH, el, {
    initialValue: `${unref(sidebar).width}px`,
  })
  const headerHeight = useCssVar(HEADER_HEIGHT, el, {
    initialValue: `${unref(header).height}px`,
  })
  const tabBarHeight = useCssVar(TAB_BAR_HEIGHT, el, {
    initialValue: `${unref(tabTar).height}px`,
  })
  const footerHeight = useCssVar(FOOTER_HEIGHT, el, {
    initialValue: `${unref(footer).height}px`,
  })

  watchEffect(() => {
    const getAsideWidth = () => {
      if (unref(isTopMenu) || !unref(sidebar).visible)
        return 0
      if (unref(getCollapsedShowTitle)) {
        return unref(menu).mixSideFixed && unref(isMixSidebar)
          ? unref(sidebar).mixSidebarWidth + unref(menu).subMenuWidth
          : unref(sidebar).mixSidebarWidth
      }
      if (unref(sidebar).collapsed) {
        return unref(menu).mixSideFixed && unref(isMixSidebar)
          ? unref(sidebar).collapsedWidth + unref(menu).subMenuWidth
          : unref(sidebar).collapsedWidth
      }
      return unref(sidebar).width
    }

    const getHeaderHeight = () => {
      if (!unref(header).visible)
        return 0
      return unref(header).height
    }

    const getTabBarHeight = () => {
      if (!unref(tabTar).visible)
        return 0
      return unref(tabTar).height
    }

    const getFooterHeight = () => {
      if (!unref(footer).visible)
        return 0
      return unref(footer).height
    }
    asideWidth.value = `${getAsideWidth()}px`
    headerHeight.value = `${getHeaderHeight()}px`
    tabBarHeight.value = `${getTabBarHeight()}px`
    footerHeight.value = `${getFooterHeight()}px`
  })
}

export function createThemeColorListen() {
  const {
    grayMode,
    colorWeak,
  } = useAppConfig()

  // const headerTextColor = useCssVar(
  //   HEADER_TEXT_COLOR_VAR,
  //   null,
  //   {
  //     initialValue: LIGHT_TEXT_COLOR,
  //   },
  // )
  // const headerActionHoverBgColor = useCssVar(
  //   HEADER_ACTION_HOVER_BG_COLOR_VAR,
  //   null,
  // )

  // const asideTextColor = useCssVar(
  //   ASIDE_TEXT_COLOR_VAR,
  //   null,
  //   {
  //     initialValue: LIGHT_TEXT_COLOR,
  //   },
  // )
  // const triggerBackgroundColor = useCssVar(
  //   TRIGGER_BG_COLOR_VAR,
  //   null,
  // )

  watchEffect(() => {
    // headerTextColor.value = pickTextColorBasedOnBgColor(
    //   unref(header).bgColor,
    //   LIGHT_TEXT_COLOR,
    //   DARK_TEXT_COLOR,
    // )

    // if (['#fff', '#ffffff'].includes(unref(header).bgColor.toLowerCase())) {
    //   headerActionHoverBgColor.value = darken(unref(header).bgColor, 6)
    //   setAppConfig({ header: { theme: ThemeEnum.LIGHT } })
    // }
    // else {
    //   headerActionHoverBgColor.value = lighten(unref(header).bgColor, 6)
    //   setAppConfig({ header: { theme: ThemeEnum.DARK } })
    // }

    // sidebarBgColor.value = unref(sidebar).bgColor
    // asideTextColor.value = pickTextColorBasedOnBgColor(
    //   unref(sidebar).bgColor,
    //   LIGHT_TEXT_COLOR,
    //   DARK_TEXT_COLOR,
    // )

    // if (['#fff', '#ffffff'].includes(unref(sidebar).bgColor.toLowerCase())) {
    //   triggerBackgroundColor.value = darken(unref(sidebar).bgColor, 6)
    //   setAppConfig({ sidebar: { theme: ThemeEnum.LIGHT } })
    // }
    // else {
    //   triggerBackgroundColor.value = lighten(unref(sidebar).bgColor, 6)
    //   setAppConfig({ sidebar: { theme: ThemeEnum.DARK } })
    // }

    toggleGrayMode(unref(grayMode))
    toggleColorWeak(unref(colorWeak))
    // toggleClass(
    //   ThemeEnum.DARK === unref(theme),
    //   ThemeEnum.DARK,
    //   document.documentElement,
    // )
  })
}

export function useAppTheme() {
  const themeStore = useThemeStore()
  const { setThemeConfig, setSidebarTheme, setHeaderTheme } = themeStore
  const { themeConfig: getThemeConfig, theme, sidebar, header } = storeToRefs(themeStore)

  const darkRef = useDark({
    selector: 'html',
    attribute: 'class',
    valueDark: 'dark',
    valueLight: 'light',
  })
  const toggleDark = useToggle(darkRef)

  const isDark = computed(() => unref(darkRef) && unref(theme) === ThemeEnum.DARK)
  const toggleTheme = (dark: boolean) => {
    const tt = (dark: boolean) => {
      theme.value = dark ? ThemeEnum.DARK : ThemeEnum.LIGHT
      toggleDark(dark)
    }

    // @ts-expect-error newapi
    if (!document.startViewTransition) {
      tt(dark)
      return
    }
    // @ts-expect-error newapi
    document.startViewTransition(() => tt(dark))
  }

  // sidebar
  const isSidebarDark = computed(() => (unref(theme) === ThemeEnum.DARK || unref(sidebar) === ThemeEnum.DARK))
  const toggleSidebarTheme = (dark: boolean) => {
    setSidebarTheme(dark ? ThemeEnum.DARK : ThemeEnum.LIGHT)
  }

  const isHeaderDark = computed(() => (unref(theme) === ThemeEnum.DARK || unref(header) === ThemeEnum.DARK))
  const toggleHeaderTheme = (dark: boolean) => {
    setHeaderTheme(dark ? ThemeEnum.DARK : ThemeEnum.LIGHT)
  }

  const primaryColor = computed(() => {
    return getThemeConfig.value.primaryColor
  })

  const infoColor = computed(() => {
    return getThemeConfig.value.infoColor
  })

  const successColor = computed(() => {
    return getThemeConfig.value.successColor
  })

  const warningColor = computed(() => {
    return getThemeConfig.value.warningColor
  })

  const errorColor = computed(() => {
    return getThemeConfig.value.errorColor
  })

  const themeColors = computed(() => {
    let colors: ThemeColors = {}
    const themeConfig = getThemeConfig.value

    if (themeConfig.primaryColor) {
      const primaryColorList = generateColors(themeConfig.primaryColor)
      colors = {
        ...colors,
        ...{
          primaryColor: primaryColorList[5],
          primaryColorHover: primaryColorList[4],
          primaryColorPressed: primaryColorList[4],
          primaryColorSuppl: primaryColorList[6],
        },
      }
    }

    if (themeConfig.infoColor) {
      const infoColorList = generateColors(themeConfig.infoColor)
      colors = {
        ...colors,
        ...{
          infoColor: infoColorList[5],
          infoColorHover: infoColorList[4],
          infoColorPressed: infoColorList[4],
          infoColorSuppl: infoColorList[6],
        },
      }
    }

    if (themeConfig.successColor) {
      const successColorList = generateColors(themeConfig.successColor)
      colors = {
        ...colors,
        ...{
          successColor: successColorList[5],
          successColorHover: successColorList[4],
          successColorPressed: successColorList[4],
          successColorSuppl: successColorList[6],
        },
      }
    }

    if (themeConfig.warningColor) {
      const warningColorList = generateColors(themeConfig.warningColor)
      colors = {
        ...colors,
        ...{
          warningColor: warningColorList[5],
          warningColorHover: warningColorList[4],
          warningColorPressed: warningColorList[4],
          warningColorSuppl: warningColorList[6],
        },
      }
    }

    if (themeConfig.errorColor) {
      const errorColorList = generateColors(themeConfig.errorColor)
      colors = {
        ...colors,
        ...{
          errorColor: errorColorList[5],
          errorColorHover: errorColorList[4],
          errorColorPressed: errorColorList[4],
          errorColorSuppl: errorColorList[6],
        },
      }
    }
    return colors
  })

  watch(
    themeColors,
    (val) => {
      val.primaryColor && setCssVar('--primary-color', val.primaryColor)
      val.successColor && setCssVar('--success-color', val.successColor)
      val.errorColor && setCssVar('--error-color', val.errorColor)
      val.warningColor && setCssVar('--warning-color', val.warningColor)
    },
    { deep: true },
  )

  return {
    isDark,
    isSidebarDark,
    isHeaderDark,
    toggleTheme,
    primaryColor,
    infoColor,
    successColor,
    warningColor,
    errorColor,
    themeColors,
    setThemeConfig,
    toggleSidebarTheme,
    toggleHeaderTheme,
  }
}
