import { acceptHMRUpdate, defineStore } from 'pinia'
import type { RouteRecordRaw } from 'vue-router/auto'
import { PAGE_NOT_FOUND_ROUTE, REDIRECT_ROUTE } from '~/modules/router/routes/basic'
import { cloneDeep, filterTree, transformRouteToMenu } from '~/utils'
import { PermissionModeEnum } from '~/constants'

export const useRouteStore = defineStore('ROUTES', () => {
  const routesRef = shallowRef<RouteRecordRaw[]>([])
  const menuListRef = ref<Menu[]>([])
  const initMenu = ref(false)

  /**
   * 初始化路由
   */
  function initRoutes(routes: RouteRecordRaw[]) {
    // 过滤配置了 meta -> ignore: true 路由
    const routeIgnoreFilter = (route: RouteRecordRaw) => {
      const { meta } = route
      const { ignore } = meta || {}
      return !ignore
    }

    // 过滤空children存在且长度为0的路由
    const cleanEmptyChildrenFilter = (route: RouteRecordRaw) => {
      return route.children ? route.children.length > 0 : true
    }

    let tmpRoutes: RouteRecordRaw[] = []
    tmpRoutes.push(PAGE_NOT_FOUND_ROUTE, REDIRECT_ROUTE)
    tmpRoutes.push(...routes)

    tmpRoutes = filterTree(tmpRoutes, routeIgnoreFilter)
    routesRef.value = filterTree(tmpRoutes, cleanEmptyChildrenFilter)

    return routesRef.value
  }

  /** 判断给定的route中是否具有给定perms列表的权限 */
  function hasPermission(route: RouteRecordRaw, perms: any[] = []) {
    if (!route.meta?.perm) {
      return true
    }

    // 递归寻找子节点perm
    if (route.meta?.perm === true && route.children?.length) {
      return filterAsyncRoutes(route.children, perms).length
    }

    // 否则直接通过perm进行判断
    return perms.includes(
      Array.isArray(route.meta?.perm)
        ? route.meta.perm[0]?.perm
        : route.meta?.perm,
    )
  }

  // 过滤掉所有perm不匹配的路由
  function filterAsyncRoutes(routes: RouteRecordRaw[], perms: any[]) {
    return routes.reduce((rs: RouteRecordRaw[], route) => {
      if (hasPermission(route, perms)) {
        rs.push({
          ...route,
          children: route.children ? filterAsyncRoutes(route.children, perms) : [],
        })
      }
      return rs
    }, [])
  }

  async function buildMenu() {
    initMenu.value = false
    const appStore = useAppConfigStore()
    const userStore = useUserStore()

    const roleList = userStore.userInfo?.roles || []
    const permList = userStore.userInfo?.perms || []
    const permissionMode = unref(appStore.permissionMode)

    const routeRoleFilter = (route: RouteRecordRaw) => {
      const { meta } = route
      const { roles } = meta || {}
      if (!roles) {
        return true
      }
      return roleList.some(role => roles.includes(role))
    }

    const r = cloneDeep(unref(routesRef))
    let menuList: Menu[] = []
    switch (permissionMode) {
      case PermissionModeEnum.ROLE:
        {
          const routes = filterTree(r, routeRoleFilter)
          menuList = transformRouteToMenu(routes, true)
        }
        break
      case PermissionModeEnum.PERM:
        menuList = transformRouteToMenu(filterAsyncRoutes(r, permList), true)
        break
    }
    menuList.sort((a, b) => {
      return (a.meta?.order || 0) - (b.meta?.order || 0)
    })
    menuListRef.value = menuList
    initMenu.value = true
  }

  return {
    routesRef,
    menuListRef,
    initMenu,
    initRoutes,
    buildMenu,
  }
}, {
  persist: {
    paths: ['menuListRef'],
  },
})

if (import.meta.hot)
  import.meta.hot.accept(acceptHMRUpdate(useRouteStore, import.meta.hot))
